/*-
 * ============LICENSE_START=======================================================
 *  Copyright (C) 2020 Nordix Foundation.
 * ================================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * SPDX-License-Identifier: Apache-2.0
 * ============LICENSE_END=========================================================
 */
package org.onap.so.etsi.nfvo.ns.lcm.app;

import static org.slf4j.LoggerFactory.getLogger;
import java.util.concurrent.Executor;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

/**
 * @author Waqas Ikram (waqas.ikram@est.tech)
 *
 */
@Configuration
@EnableAsync
public class AsyncThreadExecutorConfiguration {
    private static final Logger logger = getLogger(AsyncThreadExecutorConfiguration.class);

    @Value("${mso.async.core-pool-size:20}")
    private int corePoolSize;

    @Value("${mso.async.max-pool-size:30}")
    private int maxPoolSize;

    @Value("${mso.async.queue-capacity:50}")
    private int queueCapacity;

    @Bean
    @Primary
    public Executor asyncExecutor() {
        logger.info("Setting ThreadPoolTaskExecutor for async calls ...");
        final ThreadPoolTaskExecutor executor = new ThreadPoolTaskExecutor();
        executor.setCorePoolSize(corePoolSize);
        executor.setMaxPoolSize(maxPoolSize);
        executor.setQueueCapacity(queueCapacity);
        executor.setThreadNamePrefix("Async Process-");
        executor.initialize();
        return executor;
    }


}
